#!/usr/bin/env python3
import tempfile
from flask import request, jsonify, current_app, abort, Response
from flask_classful import FlaskView, route
from model import db, Default, AIModel
from minio.error import BucketAlreadyExists, BucketAlreadyOwnedByYou, ResponseError, NoSuchKey
from schemas import AIModelSchema, DefaultSchema
from utils import json_required, storage


class ModelView(FlaskView):
    aimodel_schema = AIModelSchema(many=False)
    default_schema = DefaultSchema(many=False)

    @json_required
    def post(self):
        return jsonify({})

    def get(self, _id: str):

        if _id == "$default":
            default = Default.query.first_or_404()  # TODO: Kitalálni, hogy inkább a latestestest-el térjen-e vissza
            m = default.default
        else:
            m = AIModel.query.fiter_by(id=_id).first_or_404()

        try:
            data = storage.connection.get_object(current_app.config['MINIO_BUCKET_NAME'], m.id)
        except NoSuchKey:
            abort(500, "The ID is stored in the database but not int the Object Store")

        return Response(data.stream(), mimetype=data.headers['Content-type'])

    @route('<_id>/details')
    def get_details(self, _id: str):

        if _id == "$default":
            default = Default.query.first_or_404()  # TODO: Kitalálni, hogy inkább a latestestest-el térjen-e vissza
            m = default.default
        else:
            m = AIModel.query.fiter_by(id=_id).first_or_404()

        return jsonify(self.aimodel_schema.dump(m))

    def delete(self, _id: str):

        m = AIModel.query.fiter_by(id=_id).first_or_404()

        storage.connection.remove_object(current_app.config['MINIO_BUCKET_NAME'], m.id)

        db.session.delete(m)
        db.session.commit()

        return '', 204

    @json_required
    @route('$default', methods=['PUT'])
    def put_default(self):
        return jsonify({})
